/****************************************************************************************

	Balloon Bomber (MisfitMAME version)

	December, 2010 by Robbbert


*******************************************************************************************/

#include "driver.h"
#include "invaders.h"
#include "machine/mb14241.h"
#include "sound/samples.h"
#include "sound/discrete.h"


/*************************************
 *
 *  Video Routines
 *
 *************************************/


static PALETTE_INIT( ballbomb )
{
	UINT8 i;
	for (i = 0; i < 8; i++)
		palette_set_color(machine, i, MAKE_RGB(pal1bit(i >> 0), pal1bit(i >> 2), pal1bit(i >> 1)));
}

INLINE void set_pixel(bitmap_t *bitmap, UINT8 y, UINT8 x, UINT8 color)
{
	if (y >= INVADERS_VCOUNTER_START_NO_VBLANK)
	{
		if (flipscreen)
			*BITMAP_ADDR16(bitmap, INVADERS_VBSTART - 1 - (y - INVADERS_VCOUNTER_START_NO_VBLANK), INVADERS_HPIXCOUNT - 1 - x) = color;
		else
			*BITMAP_ADDR16(bitmap, y - INVADERS_VCOUNTER_START_NO_VBLANK, x) = color;
	}
}


static VIDEO_UPDATE( ballbomb )
{
	pen_t pens[8];
	offs_t offs;
	UINT8 i, *color_map_base, *prom;

	for (i = 0; i < 8; i++)
		pens[i] = MAKE_RGB(pal1bit(i >> 0), pal1bit(i >> 2), pal1bit(i >> 1));

	prom = memory_region(screen->machine, "proms");
	color_map_base = player ? &prom[0x0400] : &prom[0x0000];

	for (offs = 0; offs < invaders_ram_size; offs++)
	{
		UINT8 y = offs >> 5;
		UINT8 x = offs << 3;

		offs_t color_address = (offs >> 8 << 5) | (offs & 0x1f);

		UINT8 data = invaders_ram[offs];
		UINT8 fore_color = screen_red ? 1 : color_map_base[color_address] & 0x07;

		/* blue background */
		for (i = 0; i < 8; i++)
		{
			set_pixel(bitmap, y, x|i, (data & 1) ? fore_color : 2);
			data >>= 1;
		}
	}

	return 0;
}



/*************************************
 *
 *  Sound
 *
 *************************************/

static const char *const invaders_sample_names[] =
{
	"*invaders",
	"1.wav",		/* shot/missile */
	"2.wav",		/* base hit/explosion */
	"3.wav",		/* invader hit */
	"4.wav",		/* fleet move 1 */
	"5.wav",		/* fleet move 2 */
	"6.wav",		/* fleet move 3 */
	"7.wav",		/* fleet move 4 */
	"8.wav",		/* UFO/saucer hit */
	"9.wav",		/* bonus base */
	0
};

static const samples_interface invaders_samples_interface =
{
	6,	/* 6 channels */
	invaders_sample_names
};


/*************************************
 *
 *  Discrete Sound
 * (copied from indianbt)
 *
 *************************************/

static const discrete_dac_r1_ladder ballbomb_music_dac =
	{3, {0, RES_K(47), RES_K(12)}, 0, 0, 0, CAP_U(0.1)};

#define INVADERS_MUSIC_CLK		(3993.6*2*2*2)

/* Nodes - Inputs */
#define INVADERS_MUSIC_DATA		NODE_01
/* Nodes - Sounds */
#define INVADERS_MUSIC			NODE_11

static DISCRETE_SOUND_START(ballbomb)

	DISCRETE_INPUT_DATA (INVADERS_MUSIC_DATA)

/******************************************************************************
 *
 * Music Generator
 *
 ******************************************************************************/
	DISCRETE_NOTE(NODE_20, 1, INVADERS_MUSIC_CLK, INVADERS_MUSIC_DATA, 255, 5, DISC_CLK_IS_FREQ)

	// Convert count to 7492 output
	DISCRETE_TRANSFORM2(NODE_21, NODE_20, 2, "01>0+")

	DISCRETE_DAC_R1(NODE_22, 1, NODE_21, DEFAULT_TTL_V_LOGIC_1, &ballbomb_music_dac)

/******************************************************************************
 *
 * Final Mixing and Output
 *
 ******************************************************************************/
	DISCRETE_CRFILTER(NODE_90, 1, NODE_22, RES_K(10), CAP_U(0.1))

	DISCRETE_OUTPUT(NODE_90, 21000)

DISCRETE_SOUND_END

/*************************************
 *
 *  Output Ports
 *
 *************************************/

static WRITE8_HANDLER( ballbomb_01_w )
{
	discrete_sound_w(machine, INVADERS_MUSIC_DATA, data|0x80);
}

static WRITE8_HANDLER( ballbomb_03_w )
{
	UINT8 rising_bits = data & ~port_1_last;

	if (rising_bits & 0x01) sample_start(1, 2, 0);		/* Hit a balloon */
	if (rising_bits & 0x02) sample_start(2, 0, 0);		/* Shot Sound */
	if (rising_bits & 0x04) sample_start(2, 1, 0);		/* Base Hit */
	if (rising_bits & 0x08) sample_start(1, 7, 0);		/* Hit a Bomb */
	if (rising_bits & 0x10) sample_start(3, 8, 0);		/* Bonus Base at 1500 points */

	sound_global_enable(data & 0x20);

	screen_red = data & 0x04;

	port_1_last = data;

	if (data & 0xc0) printf("3=%X ",data);
}

static WRITE8_HANDLER( ballbomb_05_w )
{
	UINT8 rising_bits = data & ~port_2_last;

	if (data & 0x01) sample_start(0, 7, 0);		/* Indicates plane will drop bombs */
	if (data & 0x04) sample_start(0, 4, 0);		/* Plane is dropping new balloons at start of level */
	if (rising_bits & 0x10) sample_start(2, 2, 0);		/* Balloon hit and bomb drops */

	player = data & 0x20;
	flipscreen = player & input_port_read(machine, "CAB");

	port_2_last = data;

	if (data & 0xca) printf("5=%X ",data);
}


/*************************************
 *
 *  Address Maps
 *
 *************************************/

static ADDRESS_MAP_START( ballbomb_map, ADDRESS_SPACE_PROGRAM, 8 )
	AM_RANGE(0x0000, 0x1fff) AM_ROM
	AM_RANGE(0x2000, 0x3fff) AM_RAM AM_BASE(&invaders_ram) AM_SIZE(&invaders_ram_size)
	AM_RANGE(0x4000, 0x5fff) AM_ROM AM_WRITENOP
ADDRESS_MAP_END

static ADDRESS_MAP_START( ballbomb_io_map, ADDRESS_SPACE_IO, 8 )
	ADDRESS_MAP_GLOBAL_MASK(0x07)
	AM_RANGE(0x01, 0x01) AM_READ_PORT("IN1")
	AM_RANGE(0x02, 0x02) AM_READ_PORT("IN2")
	AM_RANGE(0x03, 0x03) AM_READ(mb14241_0_shift_result_r)

	AM_RANGE(0x01, 0x01) AM_WRITE(ballbomb_01_w)
	AM_RANGE(0x02, 0x02) AM_WRITE(mb14241_0_shift_count_w)
	AM_RANGE(0x03, 0x03) AM_WRITE(ballbomb_03_w)
	AM_RANGE(0x04, 0x04) AM_WRITE(mb14241_0_shift_data_w)
	AM_RANGE(0x05, 0x05) AM_WRITE(ballbomb_05_w)
	AM_RANGE(0x06, 0x06) AM_WRITE(watchdog_reset_w)
ADDRESS_MAP_END

/*************************************
 *
 *  Inputs
 *
 *************************************/

static INPUT_PORTS_START( ballbomb )
	PORT_START("IN1")
	PORT_BIT( 0x01, IP_ACTIVE_LOW,  IPT_COIN1 )
	PORT_BIT( 0x02, IP_ACTIVE_HIGH, IPT_START2 )
	PORT_BIT( 0x04, IP_ACTIVE_HIGH, IPT_START1 )
	PORT_BIT( 0x08, IP_ACTIVE_HIGH, IPT_UNKNOWN )
	PORT_BIT( 0x10, IP_ACTIVE_HIGH, IPT_BUTTON1 )
	PORT_BIT( 0x20, IP_ACTIVE_HIGH, IPT_JOYSTICK_LEFT ) PORT_2WAY
	PORT_BIT( 0x40, IP_ACTIVE_HIGH, IPT_JOYSTICK_RIGHT ) PORT_2WAY
	PORT_BIT( 0x80, IP_ACTIVE_HIGH, IPT_UNKNOWN )

	PORT_START("IN2")
	PORT_DIPNAME( 0x03, 0x00, DEF_STR( Lives ) )		PORT_DIPLOCATION("SW1:1,2")
	PORT_DIPSETTING(    0x00, "3" )
	PORT_DIPSETTING(    0x01, "4" )
	PORT_DIPSETTING(    0x02, "5" )
	PORT_DIPSETTING(    0x03, "6" )
	PORT_BIT( 0x04, IP_ACTIVE_HIGH, IPT_TILT )
	PORT_DIPNAME( 0x08, 0x00, DEF_STR( Unknown ) )
	PORT_DIPSETTING(    0x08, DEF_STR( Off ) )
	PORT_DIPSETTING(    0x00, DEF_STR( On ) )
	PORT_BIT( 0x10, IP_ACTIVE_HIGH, IPT_BUTTON1 ) PORT_PLAYER(2)
	PORT_BIT( 0x20, IP_ACTIVE_HIGH, IPT_JOYSTICK_LEFT ) PORT_2WAY PORT_PLAYER(2)
	PORT_BIT( 0x40, IP_ACTIVE_HIGH, IPT_JOYSTICK_RIGHT ) PORT_2WAY PORT_PLAYER(2)
	PORT_DIPNAME( 0x80, 0x00, DEF_STR( Unknown ) )
	PORT_DIPSETTING(    0x80, DEF_STR( Off ) )
	PORT_DIPSETTING(    0x00, DEF_STR( On ) )

	INVADERS_CAB(0x20)
INPUT_PORTS_END


/*************************************
 *
 *  Machine Drivers
 *
 *************************************/

static MACHINE_DRIVER_START( ballbomb )

	/* basic machine hardware */
	MDRV_CPU_ADD("main",8080,INVADERS_CPU_CLOCK)
	MDRV_CPU_PROGRAM_MAP(ballbomb_map,0)
	MDRV_CPU_IO_MAP(ballbomb_io_map,0)
	MDRV_WATCHDOG_VBLANK_INIT(255)
	MDRV_MACHINE_START(invaders)
	MDRV_MACHINE_RESET(invaders_common)

	/* video hardware */
	MDRV_VIDEO_UPDATE(ballbomb)

	MDRV_SCREEN_ADD("main", RASTER)
	MDRV_SCREEN_FORMAT(BITMAP_FORMAT_INDEXED16)
	MDRV_SCREEN_RAW_PARAMS(INVADERS_PIXEL_CLOCK, INVADERS_HTOTAL, INVADERS_HBEND, INVADERS_HPIXCOUNT-4, INVADERS_VTOTAL, INVADERS_VBEND, INVADERS_VBSTART)

	MDRV_PALETTE_LENGTH(8)
	MDRV_PALETTE_INIT(ballbomb)

	/* sound hardware */
	MDRV_SOUND_START(samples)
	MDRV_SPEAKER_STANDARD_MONO("mono")

	MDRV_SOUND_ADD("samples", SAMPLES, 0)
	MDRV_SOUND_CONFIG(invaders_samples_interface)
	MDRV_SOUND_ROUTE(ALL_OUTPUTS, "mono", 1.0)

	MDRV_SOUND_ADD("discrete", DISCRETE, 0)
	MDRV_SOUND_CONFIG_DISCRETE(ballbomb)
	MDRV_SOUND_ROUTE(ALL_OUTPUTS, "mono", 1.0)
MACHINE_DRIVER_END


/*************************************
 *
 *  Roms
 *
 *************************************/

ROM_START( ballbomb )
	ROM_REGION( 0x10000, "main", 0 )
	ROM_LOAD( "tn01",         0x0000, 0x0800, CRC(551585b5) SHA1(7c17b046bdfca6ab107b7e68ba9bde6ca590c3d4) )
	ROM_LOAD( "tn02",         0x0800, 0x0800, CRC(7e1f734f) SHA1(a15656818cd730d9bc98d00ff1e7fe3f860bd624) )
	ROM_LOAD( "tn03",         0x1000, 0x0800, CRC(d93e20bc) SHA1(2bf72f813750cef8fad572a18fb8e9fd5bf38804) )
	ROM_LOAD( "tn04",         0x1800, 0x0800, CRC(d0689a22) SHA1(1f6b258431b7eb878853ff979e4d97a05fb6b797) )
	ROM_LOAD( "tn05-1",       0x4000, 0x0800, CRC(5d5e94f1) SHA1(b9f8ba38161ef4f0940c274e9d93fed4bb7db017) )

	ROM_REGION( 0x0800, "proms", 0 )		/* color maps player 1/player 2 */
	ROM_LOAD( "tn06",         0x0000, 0x0400, CRC(7ec554c4) SHA1(b638605ba2043fdca4c5e18755fa5fa81ed3db07) )
	ROM_LOAD( "tn07",         0x0400, 0x0400, CRC(deb0ac82) SHA1(839581c4e58cb7b0c2c14cf4f239220017cc26eb) )
ROM_END

/*************************************
 *
 *  Game Drivers
 *
 *************************************/

/* board #  rom       parent    machine   inp */

GAME( 1980, ballbomb, 0,        ballbomb, ballbomb, 0, ROT270, "Taito", "Balloon Bomber", 0)//GAME_IMPERFECT_SOUND | GAME_IMPERFECT_GRAPHICS )	/* missing clouds */
